#include <Arduino.h>
#include "controller.h"


#define DELAY_TIME 10000 // time between measurements [ms]
#define MIN_FAN_SPEED_PERCENT 24 // minimum fan speed [%]
#define MIN_TEMP 25 // turn fan off below [deg C]
#define MAX_TEMP 40 // turn fan to full speed above [deg C]

/**
 * @brief Called once at start
 * Initialize hardware pins
 */
void setup() {

  Serial.begin(115200);

  pinMode(FAN_PIN, OUTPUT);
  pinMode(SIGNAL_PIN, INPUT);
}

/**
 * @brief Get the Fan Speed in round per minutes
 * 
 * @return int 
 */
int getFanSpeedRpm() {
  int highTime = pulseIn(SIGNAL_PIN, HIGH);
  int lowTime = pulseIn(SIGNAL_PIN, LOW);
  int period = highTime + lowTime;
  if (period == 0) {
    return 0;
  }
  float freq = 1000000.0 / (float)period;
  return (freq * 60.0) / 2.0; // two cycles per revolution
}

/**
 * @brief Set the Fan Speed
 * 
 * @param p expected percentage
 */
void setFanSpeedPercent(int p) {
  int value = (p / 100.0) * 255;
  analogWrite(FAN_PIN, value);
}

/**
 * @brief Endless loop
 * (is called as fast as possible)
 */
void loop() {
  float temp = /* FIXME bme.readTemperature(); */ 
  Serial.print("Temperature is ");
  Serial.print(temp);
  Serial.println(" deg C");

  int fanSpeedPercent, actualFanSpeedRpm;

  if (temp < MIN_TEMP) {
    fanSpeedPercent = 0;
  } else if (temp > MAX_TEMP) {
    fanSpeedPercent = 100;
  } else {
    fanSpeedPercent = (100 - MIN_FAN_SPEED_PERCENT) * (temp - MIN_TEMP) / (MAX_TEMP - MIN_TEMP) + MIN_FAN_SPEED_PERCENT;
  }

  Serial.print("Setting fan speed to ");
  Serial.print(fanSpeedPercent);
  Serial.println(" %");
  setFanSpeedPercent(fanSpeedPercent);

  actualFanSpeedRpm = getFanSpeedRpm();
  Serial.print("Fan speed is ");
  Serial.print(actualFanSpeedRpm);
  Serial.println(" RPM");

  Serial.println();
  delay(DELAY_TIME);
}
